# Logit models from a Machile Learning Perspective: A classification algorithm.

library(tidyverse)
library(caret)
library(AER)

theme_set(theme_bw())

# Load the data and remove NAs
data("PimaIndiansDiabetes2", package = "mlbench")
PimaIndiansDiabetes2 <- na.omit(PimaIndiansDiabetes2)
# Inspect the data
head(PimaIndiansDiabetes2)
# Split the data into training and test set
set.seed(123)

n<-dim(PimaIndiansDiabetes2)[1]
training.samples <-sort(sample(x=1:n,size = round(0.8*n)))
train.data  <- PimaIndiansDiabetes2[training.samples, ]
test.data <- PimaIndiansDiabetes2[-training.samples, ]

# Fit the model
model <- glm( diabetes ~., data = train.data, family = binomial(link="logit"))
# Summarize the model
summary(model)
# Make predictions
predict.logit<-function(X,beta){
  X<-cbind(rep(1,dim(X)[1]),X)        
  pi<-exp(beta%*%t(X))/(1+exp(beta%*%t(X)))   
  return(pi)
}
predicted.probabilities <- predict.logit(test.data[,-dim(test.data)[2]],model$coefficients)
predicted.classes <- ifelse(predicted.probabilities > 0.5, "pos", "neg")
# Model accuracy
mean(predicted.classes == test.data$diabetes)


#Alternatively, via "caret" library we can use the createDataPartition function
training.samples <- PimaIndiansDiabetes2$diabetes %>% createDataPartition(p = 0.8, list = FALSE)
train.data  <- PimaIndiansDiabetes2[training.samples, ]
test.data <- PimaIndiansDiabetes2[-training.samples, ]
# Fit the model
model <- glm( diabetes ~., data = train.data, family = binomial)
# Summarize the model
summary(model)
# Make predictions
probabilities <- model %>% predict(test.data, type = "response")
#or
probabilities2<-predict(model,newdata=test.data, type = "response")

predicted.classes <- ifelse(probabilities > 0.5, "pos", "neg")
# Model accuracy
mean(predicted.classes == test.data$diabetes)


#Simple logistic regression
model <- glm( diabetes ~ glucose, data = train.data, family = binomial)
summary(model)$coef
coeftest(model, vcov. = vcovHC, type = "HC1")

newdata <- data.frame(glucose = c(20,  180))
probabilities <- model %>% predict(newdata, type = "response")
predicted.classes <- ifelse(probabilities > 0.5, "pos", "neg")
predicted.classes

## Plotting a fitted GLM with GG plot
train.data %>% mutate(prob = ifelse(diabetes == "pos", 1, 0)) %>%
  ggplot(aes(glucose, prob)) +
  geom_point(alpha = 0.2) +
  geom_smooth(method = "glm", method.args = list(family = "binomial")) +
  labs(
    title = "Logistic Regression Model", 
    x = "Plasma Glucose Concentration",
    y = "Probability of being diabete-pos"
  )


#Multiple logistic regression
model <- glm( diabetes ~ glucose + mass + pregnant, 
              data = train.data, family = binomial)
summary(model)$coef
coeftest(model, vcov. = vcovHC, type = "HC1")

model <- glm( diabetes ~., data = train.data, family = binomial)
summary(model)$coef
coeftest(model, vcov. = vcovHC, type = "HC1")

# Reminder: the estimated  coefficients "beta" represent 
# the expected change in log odds of having the outcome per unit change in X.

install.packages("ISLR")
library(ISLR)
head(Credit)
help(Credit)

##### Good job!